@@ -118,14 +118,15 @@ from sqlalchemy.ext.hybrid import (
 from sqlalchemy import func
 
 import json
-import re
+import pickle
+import zlib
 from pybald.db import engine, dump_engine
 from pybald.util import camel_to_underscore, pluralize
 
 import project
-from project import mc
+# from project import mc
 
-from sqlalchemy.orm.interfaces import SessionExtension
+# from sqlalchemy.orm.interfaces import SessionExtension
 from sqlalchemy.ext.mutable import Mutable
 
 from sqlalchemy import __version__ as sa_ver
@@ -135,7 +136,8 @@ sa_maj_ver, sa_min_ver, sa_rev_ver = sa_ver.split(".")
 if int(sa_min_ver) >= 7:
     from sqlalchemy.orm.attributes import flag_modified
 else:
-    from sqlalchemy.orm.attributes import instance_state, instance_dict, NO_VALUE
+    from sqlalchemy.orm.attributes import instance_dict, NO_VALUE
+
     def flag_modified(instance, key):
         state, dict_ = instance_state(instance), instance_dict(instance)
         impl = state.manager[key].impl
@@ -327,6 +329,15 @@ class JSONEncodedDict(TypeDecorator):
         return value
 
 
+class ZipPickler(object):
+    '''Simple wrapper for pickle that auto compresses/decompresses values'''
+    def loads(self, string):
+        return pickle.loads(zlib.decompress(string))
+
+    def dumps(self, obj, protocol):
+        return zlib.compress(pickle.dumps(obj, protocol))
+
+
 class MutationDict(Mutable, dict):
     '''
     A dictionary that automatically emits change events for SQA
@@ -381,6 +392,9 @@ class NotFound(NoResultFound):
     pass
 
 
+surrogate_pk_template = Column(Integer, nullable=False, primary_key=True)
+
+
 class ModelMeta(sqlalchemy.ext.declarative.DeclarativeMeta):
     '''
     MetaClass that sets up some common behaviors for pybald models.
@@ -404,7 +418,7 @@ class ModelMeta(sqlalchemy.ext.declarative.DeclarativeMeta):
         # set the tablename, if it's user set, use that, otherwise use a
         # function to create one
         cls.__tablename__ = getattr(cls, "__tablename__",
-                                        pluralize(camel_to_underscore(name)))
+                                    pluralize(camel_to_underscore(name)))
         # tableargs adds autoload to create schema reflection
         cls.__table_args__ = getattr(cls, "__table_args__", {})
 
@@ -417,21 +431,40 @@ class ModelMeta(sqlalchemy.ext.declarative.DeclarativeMeta):
         # check if the class has at least one primary key
         # if not, automatically generate one.
         has_primary = any([False] + [value.primary_key
-                                          for value in cls.__dict__.values()
-                                    if isinstance(value, Column)])
+                                     for value in cls.__dict__.values()
+                                     if isinstance(value, Column)])
         if not has_primary:
-            cls.id = Column(Integer, nullable=False, primary_key=True)
+            # treat the id as a mixin w/copy
+            surrogate_pk = surrogate_pk_template.copy()
+            surrogate_pk._creation_order = surrogate_pk_template._creation_order
+            cls.id = surrogate_pk
         super(ModelMeta, cls).__init__(name, bases, ns)
 
 
-class NonDbModel(object):
+class RegistryMount(type):
+    '''
+    A registry creating metaclass that keeps track of all defined classes that
+    inherit from a base class using this metaclass.
+    '''
+    # lifted almost verbatim from: http://martyalchin.com/2008/jan/10/simple-plugin-framework/
+    def __init__(cls, name, bases, attrs):
+        try:
+            cls.registry.append(cls)
+        except AttributeError:
+            # this is processing the first class (the mount point)
+            cls.registry = []
+
+        return super(RegistryMount, cls).__init__(name, bases, attrs)
+
+
+class RegistryModelMeta(RegistryMount, ModelMeta):
     pass
 
+
 class Model(Base):
     '''Pybald Model class, inherits from SQLAlchemy Declarative Base.'''
-    __metaclass__ = ModelMeta
+    __metaclass__ = RegistryModelMeta
 
-    # exception aliases
     NotFound = sqlalchemy.orm.exc.NoResultFound
     MultipleFound = sqlalchemy.orm.exc.MultipleResultsFound
 
@@ -565,7 +598,6 @@ class Model(Base):
         '''
         return session.query(cls).filter(*pargs, **kargs)
 
-
     @classmethod
     def query(cls):
         '''
@@ -582,3 +614,13 @@ class Model(Base):
         '''
         cls.__table__.create(dump_engine)
 
+
+class NonDbModel(object):
+    '''
+    A plain Python object that uses the RegistryMount system to register
+    non database models. Inheriting from this allows the object to be registered
+    as a Model.
+    '''
+    __metaclass__ = RegistryMount
+    # use the same registry space for NonDbModels
+    registry = Model.registry
